<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;

class InsNrMapaResECF {
    
    protected $entityManager;
    protected $params = array();
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function stNrMapaResECF($conditions,$rows){
	    $this->setParams($conditions);
	    $this->executar();
	    $this->executarNrMapa($rows);
	}
	
	private function executarNrMapa($rows){
		foreach ($rows as $row) {
			$params = array(
				'CDFILIAL'=>$row['CDFILIAL'],
				'DATA'=> $row['DTVENDA']
			);
			$this->adqNrMapa($params);
		}
	}
	
	private function adqNrMapa($params){
		$adqNrMapa = $this->executaQuery(self::ADQ_NR_MAPA,$params)->fetchAll();
		if($adqNrMapa){
			$stNrMapaResECF = $adqNrMapa[0]['NRMAPARESECF'];
		}else{
			$retorno = $this->executaQuery(self::MAX_NR_MAPA,$params)->fetchAll();
			if(empty($retorno[0]['NRMAPARESECF'])){
				$stNrMapaResECF = $this->formataStNrMapaResECF(0);
			}else{
				$stNrMapaResECF = str_pad($retorno[0]['NRMAPARESECF'], 10, "0", STR_PAD_LEFT);
			}
			$paramsNrMapa = array(
				':CDFILIAL'=> $params['CDFILIAL'],
				':NRMAPARESECF' => $stNrMapaResECF,
				':DTMAPARESECF' =>  $params['DATA']
			);
			$this->insertNrMapa($paramsNrMapa);
		}
		$this->adqExisteNrMapa($params,$stNrMapaResECF);
		$this->updateReducaoZ($params,$stNrMapaResECF);
	}
	
	private function updateReducaoZ($params,$stNrMapaResECF){
		$params = array(
			'P_CDFILIAL'=> $params['CDFILIAL'],
			'P_NRMAPARESECF' => $stNrMapaResECF,
			'P_DTVENDA' =>  $params['DATA']
		);
		
		$this->executaQuery(self::ACM_INSERE_NR_RED_Z,$params);
	}
	
	private function adqExisteNrMapa($params,$stNrMapaResECF){
		$params = array(
			'CDFILIAL'=>$params['CDFILIAL'],
			'NRMAPA'=>$stNrMapaResECF
		);
		$adqExisteNrMapa = $this->executaQuery(self::ADQ_EXITE_NR_MAPA,$params)->fetchAll();
		if(!count($adqExisteNrMapa)){
			$paramsNrMapa = array(
				':CDFILIAL'=> $params['CDFILIAL'],
				':NRMAPARESECF' => $stNrMapaResECF,
				':DTMAPARESECF' =>  $params['DATA']
			);
			$this->insertNrMapa($paramsNrMapa);
		}
	}
	
	private function setParams($conditions){
	    $this->params =  array(
	        'P_CDFILIAL'=>$conditions['P_CDFILIAL'],
	        'P_TCDFILIAL'=>$conditions['P_TCDFILIAL'],
	        'P_DTFINREDUCAOZ' => $conditions['P_DTVENDAFIN']
	    );
	}
	
	private function executar(){
	    $cdFiliais = $this->executaQuery(self::QRY_PES_FILIAL,$this->params)->fetchAll();
	    foreach ($cdFiliais as $key=>$cdFilial){
	    	$retorno = $this->executaQuery(self::MAX_NR_MAPA,$cdFilial)->fetchAll();
	    	$params = array(
	    		'CDFILIAL'=>$cdFilial['CDFILIAL'],
	    		'DTFINREDUCAOZ' => $this->params['P_DTFINREDUCAOZ']
	    	);
	    	if($retorno){
	    		$params['DTINIREDUCAOZ'] = $this->formatarData($retorno[0]['DTMAPARESECF']);
	    		$stNrMapaResECF = $retorno[0]['NRMAPARESECF'];
	    	}else{
	    		$params['DTINIREDUCAOZ'] =  '30/12/1899';
	    		$stNrMapaResECF = 0;
	    	}
	    	
	    	$this->executarReducaoZ($params,$stNrMapaResECF);
	    }
	}
	
	private function formatarData ($data){
		$data = str_replace('/','-',$data);
		$data = date("d-m-Y", strtotime($data." + 1 day"));
		$data = str_replace('-','/',$data);
		return $data;
	}
	
	private function executarReducaoZ($params,$stNrMapaResECF){
		$retorno = $this->executaQuery(self::AUX_REDUCAO_Z,$params)->fetchAll();
		if($retorno){
			$stNrMapaResECF = $this->formataStNrMapaResECF($stNrMapaResECF);
			$params = array(
				':CDFILIAL'=> $retorno[0]['CDFILIAL'],
				':NRMAPARESECF' => $stNrMapaResECF,
				':DTMAPARESECF' =>  $retorno[0]['DTREDUCAOZ']
			);
			$this->executaQuery(self::INSERE_NR_MAPA,$params);
		}
	}
	
	/*private function insertNrMapa ($params){
		$this->entityManager->getConnection()->executeQuery(self::INSERE_NR_MAPA,$params);
	}*/
	
	private function formataStNrMapaResECF($stNrMapaResECF){
		$stNrMapaResECF = (int) $stNrMapaResECF + 1;
		return str_pad($stNrMapaResECF, 10, "0", STR_PAD_LEFT);
	}
	
	private function executaQuery ($query,$params){
		return $this->entityManager->getConnection()->executeQuery($query,$params);
	}
	
	const QRY_PES_FILIAL = "
    SELECT CDFILIAL
    FROM FILIAL
    WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0)
    OR (:P_TCDFILIAL                      = 'T'))
	";
	
	const MAX_NR_MAPA = "
	SELECT DTECF.NRMAPARESECF,
	  DTECF.DTMAPARESECF,
    DTECF.CDFILIAL
	FROM MAPARESUMECF DTECF
	WHERE DTECF.CDFILIAL   = :CDFILIAL
	AND DTECF.NRMAPARESECF =
	  (SELECT MAX (NRECF.NRMAPARESECF) AS NRMAPARESECF
	  FROM MAPARESUMECF NRECF
	  WHERE NRECF.CDFILIAL = DTECF.CDFILIAL)
	";
	
	const AUX_REDUCAO_Z = "
		SELECT DISTINCT CDFILIAL,
		  DTREDUCAOZ
		FROM REDUCAOZ
		WHERE CDFILIAL = :CDFILIAL
		AND DTREDUCAOZ BETWEEN :DTINIREDUCAOZ AND :DTFINREDUCAOZ
		ORDER BY DTREDUCAOZ
	";
	
	const INSERE_NR_MAPA ="
	INSERT INTO MAPARESUMECF (CDFILIAL, NRMAPARESECF, DTMAPARESECF)
	VALUES
	(:CDFILIAL, :NRMAPARESECF, :DTMAPARESECF)
	";
	
	const ADQ_NR_MAPA = "
		SELECT CDFILIAL,
		  NRMAPARESECF,
		  DTMAPARESECF
		FROM MAPARESUMECF
		WHERE (CDFILIAL          = :CDFILIAL )
		AND (TRUNC(DTMAPARESECF) = :DATA)
	";
	
	const ADQ_EXITE_NR_MAPA = "
		SELECT NRMAPARESECF FROM MAPARESUMECF
		WHERE CDFILIAL   = :CDFILIAL
		AND NRMAPARESECF = :NRMAPA
	";
	
	const ACM_INSERE_NR_RED_Z = "
	UPDATE REDUCAOZ
      SET NRMAPARESECF = :P_NRMAPARESECF
      WHERE CDFILIAL   = :P_CDFILIAL  AND
            DTREDUCAOZ = :P_DTVENDA
	";
}