<?php
namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\NovoCodigo;

class PrevisaoEstoqueService {
    
    protected $conn;

    public function __construct(Connection $conn) {
        $this->conn = $conn;
    }
    public function getIDPREVCONSPDV($rows){
        
        $params = array (
            ':CDFILIAL' => $rows['CDFILIAL'],
            ':NRORG'    => $rows['NRORG']
            );
        return  $this->conn->fetchAssoc(self::GET_IDPREVCONSPDV,$params);
    }
    
    public function getIDAPVCONS($rows){
        
        $params = array (
            ':CDFILIAL' => $rows['CDFILIAL'],
            ':NRORG'    => $rows['NRORG']
            );
        return  $this->conn->fetchAssoc(self::GET_IDAPVCONS,$params);
    }
    
    public function getProdPrincip($rows, $IDPREVCONSPDV){
        $params = array (
            ':CDFILIAL' => $rows['CDFILIAL'],
            ':NRORG'    => $rows['NRORG'],
            ':NRSEQPREVPDV'    => $IDPREVCONSPDV
            );
        return  $this->conn->fetchAll(self::GET_PROD_PRINCIP,$params);
    }

    
    public function saveQtdeCompra($rows){
        foreach ($rows as $row) {
            $params = array (
                ':CDFILIAL'         => $row['CDFILIAL'],
                ':NRORG'            => $row['NRORG'],
                ':CDPRODUTO'        => $row['CDPRODUTO'],
                ':DTPRODCONPDV'     => $row['DTPRODCONPDV'],
                ':QTPRODINFCOMDIA'  => floatval(str_replace(',', '.', $row['QTPRODINFCOMDIA'])),
                ':NRSEQPREVPDV'     => $row['NRSEQPREVPDV']
            );
            $return =  $this->conn->executeQuery(self::UPDATE_QTPRODINFCOMDIA,$params);
        }   
        return $return;
    }
    
    public function adqDesvioPadrao($rows, $cdproduto, $diasemana){
        
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDLOJA'       => $rows['CDLOJA'],
            ':DTINICIAL'    => $rows['VENDASDIARIAS']['start'],
            ':DTFINAL'      => $rows['VENDASDIARIAS']['end'],
            ':NRORG'        => $rows['NRORG'],
            ':CDPRODUTO'    => $cdproduto,
            ':DIASEMANA'    => $diasemana
        );
        return  $this->conn->fetchAssoc(self::GET_DESVIO_PADRAO,$params);
            
    }
    
    public function adqProdComVenda($rows){
        
        $params = self::setParamsProdVend($rows);
        return  $this->conn->fetchAll(self::ADQ_PRODCOMVENDA,$params);
    }
    
    public function adqProdComVendaMedia($rows){
        
        $params = self::setParamsProdVend($rows);
        return  $this->conn->fetchAll(self::ADQ_PRODCOMVENDA_MEDIA,$params);
    }
    
    public function adqProdComVendaMediaDesvio($rows){
        
        $params = self::setParamsProdVend($rows);
        return  $this->conn->fetchAll(self::ADQ_PRODCOMVENDA_MEDIA_DESVIO,$params);
    }
    
    public function adqProdComVendaSazonalidade($rows){
        
        $params = self::setParamsProdVend($rows);
        return  $this->conn->fetchAll(self::ADQ_PRODCOMVENDA_SAZONALIDADE,$params);
    }
    
    public function adqProdComVendaSazonalidadeHora($rows){
        
        $params = self::setParamsProdVend($rows);
        return  $this->conn->fetchAll(self::ADQ_PRODCOMVENDA_SAZONALIDADE_HORA,$params);
    }
    
    private function setParamsProdVend($rows){
        return array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDLOJA'       => $rows['CDLOJA'],
            ':DTINICIAL'    => $rows['VENDASDIARIAS']['start'],
            ':DTFINAL'      => $rows['VENDASDIARIAS']['end'],
            ':NRORG'        => $rows['NRORG'],
            ':CDPRODUTO'    => isset($rows['CDPRODUTO']) && !!$rows['CDPRODUTO'] ? implode('_', $rows['CDPRODUTO']) : null,
            ':TCDPRODUTO'   => isset($rows['CDPRODUTO']) && !!$rows['CDPRODUTO'] ? 'N' : 'T'
        );
    }
    
    public function acmInsPrevisaoPdv($rows, $CODE){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDLOJA'       => $rows['CDLOJA'],
            ':NRSEQPREV'    => $CODE,
            ':CDOPERADOR'   => $rows['CDOPERADOR'],
            ':IDPREVCONPDV' => 'E',
            ':DTINIPCONPDV' => $rows['GERAPERIODO']['start'],
            ':DTFINPCONPDV' => $rows['GERAPERIODO']['end'],
            ':DTINIVCONPDV' => $rows['VENDASDIARIAS']['start'],
            ':DTFINVCONPDV' => $rows['VENDASDIARIAS']['end'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->executeQuery(self::INSERT_PREVISAO_PDV,$params);
    }
    
    public function insPrevProdHr($rows, $CODE){
        $params = self::setParamsProdVend($rows);
        $params[':NRSEQPREVPDV'] = $CODE;
        
        return $this->conn->executeQuery(self::INSERT_PREV_PROD_HR,$params);
    }
    
    public function adqMaxNrSeqPrevCons($rows, $CODE){
         $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRSEQPREVPDV'    => $CODE,
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAssoc(self::GET_MAX_SEQ_PREV_CONS,$params)['NRPRODCONPDV'];
    }
    
    public function utilRetProdAcabado($rows){
         $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchOne(self::GET_IDUTILRETPRODACABPREV,$params);
    }
 
    public function adqAuxProdLocal($rows, $produto, $utilizaRetiradaProdAcabado){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDPRODUTO'    => $produto,
            ':NRORG'        => $rows['NRORG'],
            ':DTENTRVENDA'  => $rows['GERAPERIODO']['start']
        );
    
        $query = self::GET_AUX_PROD_LOCAL;
    
        // Adiciona a verificação extra se $utilizaRetiradaProdAcabado for 'S'
        if ($utilizaRetiradaProdAcabado === 'S') {
            $query .= " AND (P2.IDUTILRETPRODACAB = 'N')";
        }
        
        return $this->conn->fetchAll($query, $params);
    }
    
    public function adqAuxProdPadrao($rows, $produto, $utilizaRetiradaProdAcabado){
         $params = array (
            ':CDPRODUTO'    => $produto,
            ':NRORG'        => $rows['NRORG'],
            ':DTENTRVENDA'  => $rows['GERAPERIODO']['start']
        );
        
        $query = self::GET_AUX_PROD_PADRAO;
    
        // Adiciona a verificação extra se $utilizaRetiradaProdAcabado for 'S'
        if ($utilizaRetiradaProdAcabado === 'S') {
            $query .= " AND (P2.IDUTILRETPRODACAB = 'N')";
        }
        
        return $this->conn->fetchAll($query, $params);
    }
    
    public function adqPrevConDia($rows, $produto, $produtoPai, $NEWCODE){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDPRODUTO'    => $produto,
            ':CDPRODUTOPAI' => $produtoPai,
            ':NRSEQPREVPDV' => $NEWCODE,
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAssoc(self::GET_PREV_CON_DIA,$params);
    }
    
    public function insertPrevConsPdv($rows, $produto, $produtoPai, $NEWCODE, $stSeqItPrevCons, $QTCONSUMO, $QTSUG, $QCOMPRA){
      
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDPRODUTO'    => $produto,
            ':CDPRODPAI'    => $produtoPai,
            ':NRSEQPREV'    => $NEWCODE,
            ':NRORG'        => $rows['NRORG'],
            ':QTPRODPDV'    => $QTCONSUMO,
            ':QTPRODINF'    => $QTSUG,
            ':QTPRODINFCOM' => $QCOMPRA,
            ':NRSEQPROD'    => str_pad($stSeqItPrevCons, 5, "0", STR_PAD_LEFT)
        );
      
        return $this->conn->executeQuery(self::INS_PREV_CON,$params);
    }
    
    public function updatePrevConsPdv($rows, $produto, $NEWCODE, $QTCONSUMO, $qtdeVend){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRSEQPREVPDV' => $NEWCODE,
            ':CDPRODUTO'    => $produto,
            ':NRORG'        => $rows['NRORG'],
            ':QTDE'         => $QTCONSUMO + $qtdeVend
        );
        
        return $this->conn->executeQuery(self::UPD_PREV_CON,$params);
    }
    
    public function updateStatus($rows, $NEWCODE, $produto, $IDEXIGAPVCONS){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRSEQPREVPDV' => $NEWCODE,
            ':CDPRODUTO'    => $produto,
            ':NRORG'        => $rows['NRORG'],
            ':IDAPVCONS'    => ($IDEXIGAPVCONS === 'S') ? 'N':'S'
        );
        
        return $this->conn->executeQuery(self::UPD_STATUS,$params);
    }
    
    public function adqDadosDiaRep($rows, $produto, $NEWCODE, $stSeqItPrevDia, $dtConsumo){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRSEQPREVPDV' => $NEWCODE,
            ':CDPRODUTO'    => $produto,
            ':NRORG'        => $rows['NRORG'],
            ':NRPRODCONPDV'    => str_pad($stSeqItPrevDia, 5, "0", STR_PAD_LEFT),
            ':DTPRODCONPDV' => $dtConsumo
        );
        return $this->conn->fetchAssoc(self::GET_DADOS_DIA_REP,$params);
    }
    
    public function insertPrevConsDia($rows, $produto, $prodPai, $NEWCODE, $stSeqItPrevDia, $QTCONSUMO, $QTSUG, $QTCOMPRAR, $dtConsumo){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDPRODUTO'    => $produto,
            ':CDPRPAIPCPDV' => $prodPai,
            ':NRSEQPREVPDV' => $NEWCODE,
            ':QTPRODCONPDVDIA'  => $QTCONSUMO,
            ':QTPRODCONINFDIA'  => $QTSUG,
            ':QTPRODINFCOMDIA'  => $QTCOMPRAR,
            ':NRPRODCONPDV'    => str_pad($stSeqItPrevDia, 5, "0", STR_PAD_LEFT),
            ':NRORG'        => $rows['NRORG'],
            ':DTPRODCONPDV' => $dtConsumo
        );
        
        return $this->conn->executeQuery(self::INS_PREV_CONS_DIA,$params);
    }
    
    public function updatePrevConsDia($rows, $produto, $NEWCODE, $stSeqItPrevDia, $QTCONSUMO, $QTPRODCONPDVDIA,$dtConsumo){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':CDPRODUTO'    => $produto,
            ':CDPRPAIPCPDV' => NULL,
            ':NRPRODCONPDV' => str_pad($stSeqItPrevDia, 5, "0", STR_PAD_LEFT),
            ':NRSEQPREVPDV' => $NEWCODE,
            ':QTDE'         => $QTCONSUMO + $QTPRODCONPDVDIA,
            ':NRORG'        => $rows['NRORG'],
            ':DTPRODCONPDV' => $dtConsumo
        );
        return $this->conn->executeQuery(self::UPD_PREV_CONS_DIA,$params);
    }
    
    public function adqPrevisaoArre($rows, $NEWCODE){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG'],
            ':NRSEQPREVPDV' => $NEWCODE
        );
        return $this->conn->fetchAll(self::GET_PREV_ARRE,$params);
    }
    
    public function updateQtde($rows, $NEWCODE, $previsao, $QTDCONSUMIDA){
        $params = array (
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG'],
            ':NRSEQPREVPDV' => $NEWCODE,
            ':CDPRODUTO'    => $previsao['CDPRODUTO'],
            ':CDPRPAIPCPDV' => $previsao['CDPRPAIPCPDV'],
            ':QTPRODCONPDV' => $QTDCONSUMIDA,
            ':QTPRODCONINF' => $previsao['QTPRODCONPDVDIA'],
            ':QTPRODINFCOM' => $previsao['QTPRODCONPDVDIA']
        );
        return $this->conn->executeQuery(self::UPD_QTDE,$params);
    }
    
    public function updatePrevConsPai ($rows, $produto, $prodPai, $NEWCODE, $doQtdeProd, $qtdeVend){
        $params = array(    
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG'],
            ':CDPRODUTO'    => $produto,
            ':CDPRPAIPCPDV' => $prodPai,
            ':NRSEQPREVPDV' => $NEWCODE,
            ':QTDE'         => $doQtdeProd + $qtdeVend
        );
        return $this->conn->executeQuery(self::UPD_PREV_CONS_PAI,$params);
    }
    
    public function getDiaEspecial ($rows){
        $feriado= [];
        $params = array(    
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG'],
            ':DTINI'        => $rows['GERAPERIODO']['start'],
            ':DTFIN'        => $rows['GERAPERIODO']['end']
        );
        
        $arrayFeriado =  $this->conn->fetchAll(self::GET_DIAESPECIAL,$params);
        foreach($arrayFeriado as $diaFeriado){
            $feriado[$diaFeriado['DATAESP']] = $diaFeriado['PERCCORRECAO'];
        }
        return $feriado;
    }
    
    public function checkProdAprov($CDFILIAL, $NRSEQPREVPDV, $NRORG){
        $params = array(    
            ':CDFILIAL'     => $CDFILIAL,
            ':NRORG'        => $NRORG,
            ':NRSEQPREVPDV' => $NRSEQPREVPDV
        );
        
        return  $this->conn->fetchAll(self::GET_PROD_APROV,$params);
    }
    
    public function delPrevisao($CDFILIAL, $NRSEQPREVPDV, $NRORG){
        $params = array(    
            ':CDFILIAL'     => $CDFILIAL,
            ':NRORG'        => $NRORG,
            ':NRSEQPREVPDV' => $NRSEQPREVPDV
        );
        
        $this->conn->executeQuery(self::DEL_LOGPREVCONSPDV,$params);
        $this->conn->executeQuery(self::DEL_PREVCONSPDVDIA,$params);
        $this->conn->executeQuery(self::DEL_PREVCONSPDV   ,$params);
        $this->conn->executeQuery(self::DEL_PREVISAOPDV   ,$params);
    }
    
    public function getPercFeriado($CDFILIAL, $NRORG){
        $params = array(    
            ':CDFILIAL'     => $CDFILIAL,
            ':NRORG'        => $NRORG
        );
        return $this->conn->fetchAssoc(self::GET_PERCFERIADO,$params)['PERCFERIADO'];
    }
    
    public function getDiasComAbertura($rows){
        $params = array(    
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':DTINICIAL'    => $rows['VENDASDIARIAS']['start'], 
            ':DTFINAL'      => $rows['VENDASDIARIAS']['end'],
            ':NRORG'        => $rows['NRORG']
        );
        $dias =  $this->conn->fetchAll(self::GET_DIAS_ABERTURA,$params);
        $diasComAbertura = array_map( 
        function($array) { return explode(' ', $array['DTABERCAIX'])[0]; }, 
        $dias
            ); 
        return $diasComAbertura;
    }
    
    public function getTipoCusto($rows){
        $params = array(    
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->fetchAll(self::GET_TIPO_CUSTO,$params);
    }
    
    public function findProdFinal($PRODUTOS, $NRORG, $CDFILIAL, $DATA){
        $array = [];
        $prodPai = [];
        foreach ($PRODUTOS as $prod) {
            // code...
            $params = array(
                ':CDSUBPRODUTO' => $prod.'%',
                ':NRORG'  => $NRORG,
                ':CDFILIAL' => $CDFILIAL,
                ':DTENTRVENDA' => $DATA
            );
            $prodPaiL = $this->conn->fetchAll(self::GET_PRODFINAL_LOCAL,$params);
            $prodPaiP = $this->conn->fetchAll(self::GET_PRODFINAL_PADRAO,$params);
                
            foreach ($prodPaiL as $value) {
                $array[$value['CDPRODUTO']] = $value['CDPRODUTO'];
            }
            foreach ($prodPaiP as $value) {
                $array[$value['CDPRODUTO']] = $value['CDPRODUTO'];
            }
        }
        if(count($array) > 0){
            return $array;
        } else {
            return null;
        }
    }
    
    public function findProdFromSubGroup($rows){
        $params = array(
            'CDGRUPPROD' => $rows['CDGRUPPROD'],
            'CDSUBGRPROD' => $rows['CDSUBGRPROD']
        );
        $produtos = $this->conn->fetchAll(self::GET_ALLPROD_GRUPO,$params);
        foreach ($produtos as $key => $value)
            {
                $object->$key = $value;
            }
        return self::findProdFinal($produtos, $rows['NRORG'], $rows['CDFILIAL']);
    }
    
    public function updateStatusMetaPrevHist($rows){
        $params = array(    
            ':CDMETAPREV'   => $rows['CDMETAPREV'],
            ':CDFILIAL'     => $rows['CDFILIAL'],
            ':NRORG'        => $rows['NRORG']
        );
        return $this->conn->executeQuery(self::UPD_METAPREVISAOHIST,$params);
    }
    
    
    public function getHistMeta($rows){
        $params = array(
            ':CDFILIAL'  => $rows['CDFILIAL'],
            ':NRORG'     => $rows['NRORG']
        );
        $histMeta = $this->conn->fetchAssoc(self::GET_HIST_META,$params);
        return $histMeta;
    }
    
    public function controlTransaction($type){
    // controla transação com banco    
        try {
            switch ($type) {
                case 'begin':
                // inicia transação sem auto-commit
                    $this->conn->beginTransaction();
                    break;
                case 'commit':
                // salva transação
                    $this->conn->commit();
                    break;
                case 'rollback':
                // descarta transação
                    $this->conn->rollBack();
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
   
    public function getProdArv($rows){
      
        $params = array(    
            ':CDGRUPPROD'   => $rows['CDGRUPPROD'],
            ':CDSUBGRPROD'     => $rows['CDSUBGRPROD']
        );
        $resultadoProdArv = $this->conn->fetchAll(self::queryprod,$params);
        return $resultadoProdArv;
    }
    
    public function findMateriaPrima($conditions) {
        
        $params = array(
         	":CDFILIAL"       => $conditions['CDFILIAL'],
         	":CDLOJA"         => $conditions['CDLOJA'],
         	":TCDLOJA"        => $conditions['TCDLOJA'],
            ":DATAINI"          => $conditions['DATAINI'],
            ":DATAFIN"          => $conditions['DATAFIN'],
 		    );
        return $this->conn->fetchAll(self::GET_MAT_PRIMA, $params);
    }
    

    
    const queryprod = "
    SELECT 
    PO.CDARVPROD, PO.NMPRODUTO, 
    PO.CDPRODUTO, PO.CDNVPRODUTO, 
    PR.CDFILISUBGR,PR.CDSUBGRPROD,
    PR.CDGRUPPROD
        FROM PRODUTO PO, PRSUBGRPROD PR
    WHERE  PR.CDGRUPPROD  = :CDGRUPPROD
        AND  PR.CDSUBGRPROD = :CDSUBGRPROD
        AND  PR.CDPRODUTO   = PO.CDPRODUTO
    ORDER BY PO.CDARVPROD
    ";
    
    const GET_TIPO_CUSTO = "
        SELECT NVL(TO_NUMBER(CDTIPCUSTEST),3) AS TIPOCUSTO
            FROM PARAMFILIAL
           WHERE CDFILIAL = :CDFILIAL
             AND NRORG    = :NRORG
    ";
    
    const GET_ALLPROD_GRUPO = "
    SELECT A.NRORG,
      A.CDFILISUBGR,
      B.CDNVPRODUTO,
      A.CDSUBGRPROD,
      A.CDGRUPPROD,
      B.CDPRODUTO,
      B.NMPRODUTO,
      B.CDARVPROD
    FROM PRODUTO B
    JOIN NVSUBGRPROD A
    ON(B.CDPRODUTO               = A.CDPRODUTO)
    AND (NVL(B.IDPRODATIVPR,'S') = 'S')
    WHERE A.CDSUBGRPROD = :CDSUBGRPROD
    AND A.CDGRUPPROD = :CDGRUPPROD
    ORDER BY B.CDARVPROD";
    
    const GET_PRODFINAL_PADRAO = "
    SELECT DISTINCT CDPRODUTO FROM ROTFABRICA
        WHERE CDSUBPRODUTO LIKE :CDSUBPRODUTO
        AND NRORG = :NRORG
        AND (:DTENTRVENDA BETWEEN NVL(DTINIVGPRODROT, :DTENTRVENDA) AND NVL(DTFINVGPRODROT, :DTENTRVENDA))";
        
    const GET_PRODFINAL_LOCAL = "
    SELECT DISTINCT CDPRODUTO FROM ROTFABLOCAL
        WHERE CDSUBPRODLOC LIKE :CDSUBPRODUTO
        AND NRORG = :NRORG
        AND CDFILIAL = :CDFILIAL
        AND (:DTENTRVENDA BETWEEN NVL(DTINIVGPRODLOC, :DTENTRVENDA) AND NVL(DTFINVGPRODLOC, :DTENTRVENDA))";
    
    const GET_PERCFERIADO = "
    SELECT PERCFERIADO
        FROM PARAVEND
    WHERE CDFILIAL = :CDFILIAL 
        AND   NRORG = :NRORG";
    
    const DEL_LOGPREVCONSPDV = "
      DELETE FROM LOGPREVCONSPDV
    WHERE
      CDFILIAL      = :CDFILIAL AND
      NRSEQPREVPDV  = :NRSEQPREVPDV AND
      NRORG         = :NRORG";
     
    const DEL_PREVCONSPDVDIA = "
      DELETE FROM PREVCONSPDVDIA
    WHERE
      CDFILIAL      = :CDFILIAL AND
      NRSEQPREVPDV  = :NRSEQPREVPDV AND
      NRORG         = :NRORG";
      
    const DEL_PREVCONSPDV = "
      DELETE FROM PREVCONSPDV
    WHERE
      CDFILIAL      = :CDFILIAL AND
      NRSEQPREVPDV  = :NRSEQPREVPDV AND
      NRORG         = :NRORG";
      
    const DEL_PREVISAOPDV = "
      DELETE FROM PREVISAOPDV
    WHERE
      CDFILIAL      = :CDFILIAL AND
      NRSEQPREVPDV  = :NRSEQPREVPDV AND
      NRORG         = :NRORG";
    
    const GET_PROD_APROV = "
    SELECT
         CDPRODUTO
      FROM
         PREVCONSPDV
      WHERE
         (CDFILIAL        = :CDFILIAL    ) AND
         (NRORG           = :NRORG       ) AND
         (NRSEQPREVPDV    = :NRSEQPREVPDV) AND
         (IDAPROVPREVCONS <> 'N')";
    
    const GET_DIAESPECIAL = "
       SELECT CDFILIAL, TO_CHAR(DATAESP, 'DD/MM/YYYY') AS DATAESP, PERCCORRECAO
        FROM METADIAESPECIAL 
            WHERE CDFILIAL = :CDFILIAL 
                AND DATAESP BETWEEN :DTINI AND :DTFIN 
                AND NRORG = :NRORG";
    
    const UPD_PREV_CONS_PAI = "
    UPDATE PREVCONSPDV
        SET
            QTPRODCONINF = :QTDE,
            QTPRODCONPDV = :QTDE,
            QTPRODINFCOM = :QTDE
        WHERE
            CDFILIAL        = :CDFILIAL AND
            NRSEQPREVPDV    = :NRSEQPREVPDV AND
            CDPRPAIPCPDV    = :CDPRPAIPCPDV AND
            CDPRODUTO       = :CDPRODUTO AND 
            NRORG           = :NRORG
    ";
    
    const UPD_QTDE = "
    UPDATE 
       PREVCONSPDV
    SET
       QTPRODCONPDV =  :QTPRODCONPDV,
       QTPRODCONINF   =  :QTPRODCONINF,
       QTPRODINFCOM   = :QTPRODINFCOM
    WHERE
       (NRORG = :NRORG) AND
       (CDFILIAL               =  :CDFILIAL)  AND
       (NRSEQPREVPDV =  :NRSEQPREVPDV) AND
       (CDPRODUTO        =  :CDPRODUTO) AND
       ((CDPRPAIPCPDV   = :CDPRPAIPCPDV) OR
       (CDPRPAIPCPDV IS NULL))";
    
    const GET_PREV_ARRE = "
    SELECT
       CDFILIAL, NRSEQPREVPDV,CDPRODUTO, CDPRPAIPCPDV,
       SUM(QTPRODCONPDVDIA) AS QTPRODCONPDVDIA
    FROM
       PREVCONSPDVDIA
    WHERE
       CDFILIAL     = :CDFILIAL AND
       NRSEQPREVPDV = :NRSEQPREVPDV AND
       NRORG        = :NRORG
    GROUP BY
       CDFILIAL, NRSEQPREVPDV, CDPRODUTO, CDPRPAIPCPDV
    ORDER BY
       CDFILIAL, NRSEQPREVPDV, CDPRODUTO, CDPRPAIPCPDV
    ";
    
    const UPD_PREV_CONS_DIA = "
    UPDATE PREVCONSPDVDIA
      SET
      QTPRODCONINFDIA = :QTDE,
      QTPRODCONPDVDIA = :QTDE,
      QTPRODINFCOMDIA = :QTDE,
      CDPRPAIPCPDV = :CDPRPAIPCPDV
      WHERE
      CDFILIAL     = :CDFILIAL AND
      NRSEQPREVPDV = :NRSEQPREVPDV AND
      CDPRODUTO    = :CDPRODUTO    AND
      DTPRODCONPDV = :DTPRODCONPDV AND
      NRPRODCONPDV = :NRPRODCONPDV AND
      NRORG        = :NRORG";
    
    const INS_PREV_CONS_DIA = "
    INSERT INTO 
    PREVCONSPDVDIA    (CDFILIAL    , NRSEQPREVPDV, CDPRODUTO      , NRPRODCONPDV   ,     
                        DTPRODCONPDV, CDPRPAIPCPDV, QTPRODCONPDVDIA, QTPRODCONINFDIA,     QTPRODINFCOMDIA, NRORG)
        VALUES    
                        (:CDFILIAL,  :NRSEQPREVPDV, :CDPRODUTO      , :NRPRODCONPDV   , 
                        :DTPRODCONPDV, :CDPRPAIPCPDV, :QTPRODCONPDVDIA, :QTPRODCONINFDIA,    :QTPRODINFCOMDIA, :NRORG) ";
    
    const GET_DADOS_DIA_REP = "
    SELECT
         QTPRODCONPDVDIA
      FROM
         PREVCONSPDVDIA
      WHERE
         (CDFILIAL     = :CDFILIAL    ) AND
         (NRSEQPREVPDV = :NRSEQPREVPDV) AND
         (CDPRODUTO    = :CDPRODUTO   ) AND
         (NRPRODCONPDV = :NRPRODCONPDV) AND
         (DTPRODCONPDV = :DTPRODCONPDV) AND
         (NRORG        = :NRORG       )
    ";
    
    const UPD_PREV_CON = "
    UPDATE PREVCONSPDV 
        SET 
            QTPRODCONINF = :QTDE, QTPRODCONPDV = :QTDE,  QTPRODINFCOM = :QTDE 
        WHERE 
            CDFILIAL  =  :CDFILIAL AND NRSEQPREVPDV = :NRSEQPREVPDV AND CDPRODUTO   = :CDPRODUTO AND NRORG = :NRORG";
    
    const UPD_STATUS = "
    UPDATE PREVCONSPDV 
    SET 
        IDAPROVPREVCONS = :IDAPVCONS
    WHERE 
        NRORG = :NRORG
        AND CDFILIAL  =  :CDFILIAL 
        AND NRSEQPREVPDV = :NRSEQPREVPDV 
        AND (CDPRODUTO = CDPRPAIPCPDV OR CDPRPAIPCPDV IS NULL)
        AND CDPRODUTO = :CDPRODUTO";

    const INS_PREV_CON = "
    INSERT INTO PREVCONSPDV
        (CDFILIAL    , NRSEQPREVPDV, CDPRODUTO, NRPRODCONPDV,
        CDPRPAIPCPDV, QTPRODCONPDV, QTPRODCONINF, QTPRODINFCOM, NRORG) 
      VALUES   
        (:CDFILIAL , :NRSEQPREV, :CDPRODUTO, :NRSEQPROD,
        :CDPRODPAI, :QTPRODPDV, :QTPRODINF, :QTPRODINFCOM, :NRORG)";
    
    const GET_PREV_CON_DIA = "
    SELECT
         NRPRODCONPDV, QTPRODCONINF
      FROM
         PREVCONSPDV
      WHERE
         (CDFILIAL     = :CDFILIAL    ) AND
         (NRSEQPREVPDV = :NRSEQPREVPDV) AND
         (NRORG        = :NRORG) AND
         (((CDPRODUTO||CDPRPAIPCPDV = :CDPRODUTOPAI))  OR ((CDPRODUTO = :CDPRODUTO ) AND (CDPRPAIPCPDV = CDPRODUTO)))
    ";
    
    const GET_IDUTILRETPRODACABPREV = "
        SELECT IDUTILRETPRODACABPREV
        FROM PARAVEND
        WHERE CDFILIAL = :CDFILIAL
          AND NRORG    = :NRORG    
    ";
    
    const GET_AUX_PROD_LOCAL = "
    SELECT
         RL.CDFILIAL, RL.CDPRODUTO, RL.CDSUBPRODLOC AS CDSUBPRODUTO,
         RL.QTSUBPRODLOC AS QTSUBPRODROT
      FROM
         ROTFABLOCAL RL, PRODUTO P2
      WHERE
         (RL.CDFILIAL     = :CDFILIAL)  AND
         (RL.NRORG        = :NRORG)  AND
         (RL.CDPRODUTO    = :CDPRODUTO) AND
         (RL.CDPRODUTO = P2.CDPRODUTO)
         AND (:DTENTRVENDA BETWEEN NVL(DTINIVGPRODLOC, :DTENTRVENDA) AND NVL(DTFINVGPRODLOC, :DTENTRVENDA))";
         
    const GET_AUX_PROD_PADRAO = "
    SELECT
         RL.CDPRODUTO, RL.CDSUBPRODUTO, RL.QTSUBPRODROT
      FROM
         ROTFABRICA RL, PRODUTO P2
      WHERE
         (RL.CDPRODUTO = :CDPRODUTO) AND
         (RL.NRORG     = :NRORG) AND
         (RL.CDPRODUTO = P2.CDPRODUTO) 
         AND (:DTENTRVENDA BETWEEN NVL(DTINIVGPRODROT, :DTENTRVENDA) AND NVL(DTFINVGPRODROT, :DTENTRVENDA))";
    
    const GET_MAX_SEQ_PREV_CONS = "
    SELECT NRPRODCONPDV
        FROM
        (SELECT
           NVL(MAX(NRPRODCONPDV),  0) + 1 AS NRPRODCONPDV
        FROM
           PREVCONSPDV
        WHERE
           (CDFILIAL     = :CDFILIAL    ) AND
           (NRORG        = :NRORG    ) AND
           (NRSEQPREVPDV = :NRSEQPREVPDV) )A";
    
    const INSERT_PREVISAO_PDV = 
    "INSERT INTO PREVISAOPDV
       (CDFILIAL, NRSEQPREVPDV, CDLOJA, CDOPERADOR, DTPREVCONPDV, IDPREVCONPDV,DTINIPCONPDV,
        DTFINPCONPDV,DTINIVCONPDV,DTFINVCONPDV)
    VALUES
      (:CDFILIAL, :NRSEQPREV, :CDLOJA, :CDOPERADOR, TO_DATE(SYSDATE, 'DD/MM/YYYY HH24:MI:SS'), :IDPREVCONPDV,:DTINIPCONPDV,
       :DTFINPCONPDV, :DTINIVCONPDV, :DTFINVCONPDV)";
     
    const GET_IDPREVCONSPDV = 
    "SELECT IDPREVCONSPDV,  CASE WHEN IDPREVCONSPDV = '1' THEN 'Gerada com base na média aritmética de vendas' WHEN IDPREVCONSPDV = '2' THEN 'Gerada com base na quantidade vendida no dia' WHEN IDPREVCONSPDV = '3' THEN 'Gerada com base na sazonalidade' END AS NMPREVCONSPDV FROM PARAVEND 
        WHERE 
        CDFILIAL = :CDFILIAL AND
        NRORG = :NRORG ";
    
    const GET_PROD_PRINCIP = "
    SELECT A.CDPRODUTO AS CDPRODUTO,
            A.CDARVPROD AS CDARVPROD,
            A.CDPRPAIPCPDV AS CDPRPAIPCPDV
    FROM   (SELECT PC.CDPRODUTO,
                    PR.CDARVPROD,
                    PC.CDPRPAIPCPDV
            FROM   PREVCONSPDV PC,
                    PRODUTO PR
            WHERE  (PC.CDFILIAL = :CDFILIAL)
                    AND ( PC.NRSEQPREVPDV = :NRSEQPREVPDV )
                    AND ( PC.CDPRODUTO = PR.CDPRODUTO )
                    AND ( ( PC.CDPRPAIPCPDV = PC.CDPRODUTO )
                            OR ( PC.CDPRPAIPCPDV IS NULL ) )
                    AND ( PC.NRORG = :NRORG )
                    AND ( PC.NRORG = PR.NRORG )
            ORDER  BY PC.CDPRPAIPCPDV,
                        PC.CDPRODUTO,
                        PR.CDARVPROD) A,
            (SELECT CDPRODUTO AS CDPRODUTO,
                    CDARVPROD AS CDARVPROD,
                    CDPRPAIPCPDV
            FROM  (SELECT '0'             AS CDPRODUTO,
                            '0'             AS CDARVPROD,
                            PC.CDPRPAIPCPDV AS CDPRPAIPCPDV
                    FROM   PREVCONSPDV PC
                            JOIN PRODUTO PR
                            ON PR.CDPRODUTO = PC.CDPRPAIPCPDV
                                AND PR.NRORG = PC.NRORG
                            JOIN PRODFILI PF
                            ON PF.CDFILIAL = PC.CDFILIAL
                                AND PF.CDPRODUTO = PC.CDPRODUTO
                    WHERE  ( PC.CDFILIAL = :CDFILIAL )
                            AND ( PC.NRSEQPREVPDV = :NRSEQPREVPDV )
                            AND ( PC.NRORG = :NRORG )
                            AND PF.IDCNTRESTOQ LIKE 'S'
                    GROUP  BY PC.CDPRPAIPCPDV
                    ORDER  BY PC.CDPRPAIPCPDV)
            UNION ALL
            SELECT P.CDPRODUTO    AS CDPRODUTO,
                    P.CDARVPROD    AS CDARVPROD,
                    P.CDPRPAIPCPDV AS CDPRPAIPCPDV
            FROM  (SELECT PC.CDPRODUTO,
                            PR.CDARVPROD,
                            PC.CDPRPAIPCPDV
                    FROM   PREVCONSPDV PC
                            JOIN PRODUTO PR
                            ON PR.CDPRODUTO = PC.CDPRODUTO
                                AND PR.NRORG = PC.NRORG
                            JOIN PRODFILI PF
                            ON PF.CDFILIAL = PC.CDFILIAL
                                AND PF.CDPRODUTO = PC.CDPRODUTO
                    WHERE  ( PC.CDFILIAL = :CDFILIAL )
                            AND ( PC.NRSEQPREVPDV = :NRSEQPREVPDV )
                            AND ( PC.NRORG = :NRORG )
                            AND PC.CDPRPAIPCPDV IS NULL
                            AND PF.IDCNTRESTOQ LIKE 'S'
                    ORDER  BY PC.CDPRODUTO,
                                PR.CDARVPROD) P
            ORDER  BY CDPRODUTO,
                        CDARVPROD) B
    WHERE  ( A.CDPRPAIPCPDV = B.CDPRPAIPCPDV
                AND B.CDPRPAIPCPDV IS NOT NULL )
            OR ( A.CDPRODUTO = B.CDPRODUTO
                    AND B.CDPRPAIPCPDV IS NULL )"; 
        
    const GET_IDAPVCONS = 
    "SELECT IDEXIGAPROVPREV,  CASE WHEN IDEXIGAPROVPREV = 'S' THEN 'Exige aprovação' WHEN IDEXIGAPROVPREV = 'N' THEN 'Não exige aprovação' END AS NMAPVCONS FROM PARAVEND 
        WHERE 
        CDFILIAL = :CDFILIAL AND
        NRORG = :NRORG";
        
    const ADQ_PRODCOMVENDA = 
    "SELECT
   A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, TO_CHAR(A.DTENTRVENDA, 'DD/MM/YYYY') AS DTENTRVENDA,
   SUM(A.QTCONSUMO) AS QTCONSUMO, SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG, SUM(A.QTCOMPRAR) AS QTCOMPRAR
FROM
   (SELECT
       I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, V.DTENTRVENDA,
       SUM(I.QTPRODVEND) AS QTCONSUMO,
       SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
       SUM(I.QTPRODVEND) AS QTCOMPRAR
    FROM
       VENDA V, ITEMVENDA I, PRODUTO P
    WHERE
       (V.CDFILIAL    = :CDFILIAL ) AND
       (V.CDLOJA      = :CDLOJA   ) AND
       (V.NRORG       = :NRORG    ) AND
       (V.CDFILIAL    = I.CDFILIAL  ) AND
       (V.CDCAIXA     = I.CDCAIXA   ) AND
       (V.NRSEQVENDA  = I.NRSEQVENDA) AND
       (V.IDSITUVENDA = 'O'         ) AND
       (I.CDPRODUTO   = P.CDPRODUTO ) AND
       (I.IDIMPPRGRPROMO <> 'S'     ) AND
       (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
       ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
       (NOT EXISTS(SELECT 1
                     FROM ITEMVENDAEST
                    WHERE CDFILIAL     = I.CDFILIAL
                      AND CDCAIXA      = I.CDCAIXA
                      AND NRSEQVENDA   = I.NRSEQVENDA
                      AND NRSEQUITVEND = I.NRSEQUITVEND))
    GROUP BY
       I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, V.DTENTRVENDA

    UNION ALL

    SELECT
       I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, V.DTENTRVENDA,
       SUM(I.QTITVENDAEST) AS QTCONSUMO,
       SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
       SUM(I.QTITVENDAEST) AS QTCOMPRAR
    FROM
       VENDA V, ITEMVENDAEST I, PRODUTO P
    WHERE
       (V.CDFILIAL    = :CDFILIAL ) AND
       (V.CDLOJA      = :CDLOJA   ) AND
       (V.NRORG       = :NRORG   ) AND
       (V.CDFILIAL    = I.CDFILIAL  ) AND
       (V.CDCAIXA     = I.CDCAIXA   ) AND
       (V.NRSEQVENDA  = I.NRSEQVENDA) AND
       (V.IDSITUVENDA = 'O'         ) AND
       (I.CDPRODUTO   = P.CDPRODUTO ) AND
       (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
       ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T'))
    GROUP BY
       I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, V.DTENTRVENDA) A
GROUP BY
   A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA
ORDER BY
   A.DTENTRVENDA, A.CDARVPROD";
    const ADQ_PRODCOMVENDA_MEDIA = "
    SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, SUM(QTCONSUMO) QTCONSUMO, SUM(QTCOMPRARSUG) QTCOMPRARSUG, SUM(QTCOMPRAR) AS QTCOMPRAR
FROM (SELECT
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA,
           SUM(A.QTCONSUMO) AS QTCONSUMO, SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG, SUM(A.QTCOMPRAR) AS QTCOMPRAR, NRDIASEMANA, DATADAVENDA
        FROM
           (SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTPRODVEND) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
               SUM(I.QTPRODVEND) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V
               JOIN ITEMVENDA I ON
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG       = I.NRORG) 
               JOIN PRODUTO P ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND               
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)  AND
               (I.IDIMPPRGRPROMO <> 'S') AND
               (NOT EXISTS(SELECT 1
                     FROM ITEMVENDAEST
                    WHERE CDFILIAL     = I.CDFILIAL
                      AND CDCAIXA      = I.CDCAIXA
                      AND NRSEQVENDA   = I.NRSEQVENDA
                      AND NRSEQUITVEND = I.NRSEQUITVEND))
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')
        
            UNION ALL
        
            SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTITVENDAEST) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
               SUM(I.QTITVENDAEST) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V 
               JOIN  ITEMVENDAEST I ON 
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG = I.NRORG) 
               JOIN  PRODUTO P      ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND              
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)              
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')) A
        GROUP BY
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA, NRDIASEMANA, DATADAVENDA
        ORDER BY
           A.CDARVPROD)
        GROUP BY CDPRODUTO, CDARVPROD, NMPRODUTO";
const ADQ_PRODCOMVENDA_MEDIA_DESVIO = "
    SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, DTENTRVENDA, NRDIASEMANA-1 AS NRDIASEMANA, SUM(QTCONSUMO) QTCONSUMO, SUM(QTCOMPRARSUG) QTCOMPRARSUG, SUM(QTCOMPRAR) AS QTCOMPRAR
    FROM (SELECT
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA,
           SUM(A.QTCONSUMO) AS QTCONSUMO, SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG, SUM(A.QTCOMPRAR) AS QTCOMPRAR, NRDIASEMANA, DATADAVENDA
        FROM
           (SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTPRODVEND) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
               SUM(I.QTPRODVEND) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V
               JOIN ITEMVENDA I ON
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG       = I.NRORG) 
               JOIN PRODUTO P ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND               
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG) AND
               (I.IDIMPPRGRPROMO <> 'S'     ) AND
               (NOT EXISTS(SELECT 1
                     FROM ITEMVENDAEST
                    WHERE CDFILIAL     = I.CDFILIAL
                      AND CDCAIXA      = I.CDCAIXA
                      AND NRSEQVENDA   = I.NRSEQVENDA
                      AND NRSEQUITVEND = I.NRSEQUITVEND))            
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')
        
            UNION ALL
        
            SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTITVENDAEST) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
               SUM(I.QTITVENDAEST) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V 
               JOIN  ITEMVENDAEST I ON 
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG = I.NRORG) 
               JOIN  PRODUTO P      ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND              
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)              
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')) A
        GROUP BY
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA, NRDIASEMANA, DATADAVENDA
        ORDER BY
           A.CDARVPROD)
           GROUP BY CDPRODUTO, CDARVPROD, NRDIASEMANA, DTENTRVENDA, NMPRODUTO";
    
    const ADQ_PRODCOMVENDA_SAZONALIDADE = "
    SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, DTENTRVENDA, NRDIASEMANA, SUM(QTCONSUMO) QTCONSUMO, SUM(QTCOMPRARSUG) QTCOMPRARSUG, SUM(QTCOMPRAR) AS QTCOMPRAR, NRSEMANA, DATADAVENDA
        FROM (SELECT
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA,
           SUM(A.QTCONSUMO) AS QTCONSUMO, SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG, SUM(A.QTCOMPRAR) AS QTCOMPRAR, NRDIASEMANA, DATADAVENDA, CASE  WHEN TO_CHAR(DATADAVENDA, 'DD') <= 7 THEN '1' WHEN TO_CHAR(DATADAVENDA, 'DD') > 7 AND TO_CHAR(DATADAVENDA, 'DD') <= 14 THEN '2' WHEN TO_CHAR(DATADAVENDA, 'DD') > 14 AND TO_CHAR(DATADAVENDA, 'DD') <= 21 THEN '3' WHEN TO_CHAR(DATADAVENDA, 'DD') > 21 AND TO_CHAR(DATADAVENDA, 'DD') <= 28 THEN '4' WHEN TO_CHAR(DATADAVENDA, 'DD') > 28 THEN '5' END AS NRSEMANA
        FROM
           (SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTPRODVEND) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
               SUM(I.QTPRODVEND) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V
               JOIN ITEMVENDA I ON
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG       = I.NRORG) 
               JOIN PRODUTO P ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND               
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG) AND
               (I.IDIMPPRGRPROMO <> 'S') AND
               (NOT EXISTS(SELECT 1
                     FROM ITEMVENDAEST
                    WHERE CDFILIAL     = I.CDFILIAL
                      AND CDCAIXA      = I.CDCAIXA
                      AND NRSEQVENDA   = I.NRSEQVENDA
                      AND NRSEQUITVEND = I.NRSEQUITVEND))              
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')
        
            UNION
        
            SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTITVENDAEST) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
               SUM(I.QTITVENDAEST) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V 
               JOIN  ITEMVENDAEST I ON 
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG = I.NRORG) 
               JOIN  PRODUTO P      ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND              
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)              
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')) A
        GROUP BY
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA, NRDIASEMANA, DATADAVENDA
        ORDER BY
           A.CDARVPROD)
           GROUP BY CDPRODUTO, CDARVPROD, NRSEMANA, NRDIASEMANA, DTENTRVENDA, NMPRODUTO, DATADAVENDA
           ORDER BY CDARVPROD, NRDIASEMANA, NRSEMANA";
    
    const GET_DESVIO_PADRAO = "
     SELECT MAX(QTCONSUMO) as MAXQTCONSUMO, MIN(QTCONSUMO) AS MINQTCONSUMO, COUNT(CDPRODUTO) AS QTPRODDIASEMANA FROM (SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTPRODVEND) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
               SUM(I.QTPRODVEND) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA
            FROM
               VENDA V, ITEMVENDA I, PRODUTO P
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND
               (V.CDFILIAL    = I.CDFILIAL  ) AND
               (V.CDCAIXA     = I.CDCAIXA   ) AND
               (V.NRSEQVENDA  = I.NRSEQVENDA) AND
               (V.IDSITUVENDA = 'O'         ) AND
               (I.CDPRODUTO   = P.CDPRODUTO ) AND
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               (P.CDPRODUTO = :CDPRODUTO) AND
               TO_CHAR(DTENTRVENDA, 'D') = :DIASEMANA and
               (V.NRORG = :NRORG) AND
               (V.NRORG = I.NRORG) AND 
               (V.NRORG = P.NRORG)
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D')) 
                ";
                
    const UPDATE_QTPRODINFCOMDIA = "
    UPDATE PREVCONSPDVDIA 
        SET QTPRODINFCOMDIA = :QTPRODINFCOMDIA
        WHERE CDFILIAL = :CDFILIAL 
            AND NRORG = :NRORG
            AND NRSEQPREVPDV = :NRSEQPREVPDV
            AND DTPRODCONPDV = :DTPRODCONPDV
            AND CDPRODUTO    = :CDPRODUTO
    ";
    
    const GET_DIAS_ABERTURA = 
    "SELECT TRUNC(DTABERCAIX) AS DTABERCAIX 
        FROM TURCAIXA 
        WHERE CDFILIAL = :CDFILIAL AND
             TRUNC(DTABERCAIX) BETWEEN :DTINICIAL AND :DTFINAL 
             GROUP BY  TRUNC(DTABERCAIX) 
             ORDER BY DTABERCAIX";
             
             
    const ADQ_PRODCOMVENDA_SAZONALIDADE_HORA = 
    "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, DTENTRVENDA, NRDIASEMANA, SUM(QTCONSUMO) QTCONSUMO, SUM(QTCOMPRARSUG) QTCOMPRARSUG, SUM(QTCOMPRAR) AS QTCOMPRAR, NRSEMANA, TO_CHAR(DTVENDA, 'HH24') AS HORA
        FROM (SELECT
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA,
           SUM(A.QTCONSUMO) AS QTCONSUMO, SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG, SUM(A.QTCOMPRAR) AS QTCOMPRAR, NRDIASEMANA, DATADAVENDA, CASE  WHEN TO_CHAR(DATADAVENDA, 'DD') <= 7 THEN '1' WHEN TO_CHAR(DATADAVENDA, 'DD') > 7 AND TO_CHAR(DATADAVENDA, 'DD') <= 14 THEN '2' WHEN TO_CHAR(DATADAVENDA, 'DD') > 14 AND TO_CHAR(DATADAVENDA, 'DD') <= 21 THEN '3' WHEN TO_CHAR(DATADAVENDA, 'DD') > 21 AND TO_CHAR(DATADAVENDA, 'DD') <= 28 THEN '4' WHEN TO_CHAR(DATADAVENDA, 'DD') > 28 THEN '5' END AS NRSEMANA,
           DTVENDA
        FROM
           (SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTPRODVEND) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
               SUM(I.QTPRODVEND) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA,
               V.DTVENDA
            FROM
               VENDA V
               JOIN ITEMVENDA I ON
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG       = I.NRORG) 
               JOIN PRODUTO P ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND               
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)             
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D'), V.DTVENDA
        
            UNION
        
            SELECT
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, TO_DATE(:DTINICIAL) AS DTENTRVENDA,
               SUM(I.QTITVENDAEST) AS QTCONSUMO, DTENTRVENDA DATADAVENDA,
               SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
               SUM(I.QTITVENDAEST) AS QTCOMPRAR, TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA,
               V.DTVENDA
            FROM
               VENDA V 
               JOIN  ITEMVENDAEST I ON 
                   (V.CDFILIAL    = I.CDFILIAL  ) AND
                   (V.CDCAIXA     = I.CDCAIXA   ) AND
                   (V.NRSEQVENDA  = I.NRSEQVENDA) AND
                   (V.NRORG = I.NRORG) 
               JOIN  PRODUTO P      ON
                   (I.CDPRODUTO   = P.CDPRODUTO ) AND
                   (V.NRORG = P.NRORG)
            WHERE
               (V.CDFILIAL    = :CDFILIAL ) AND
               (V.CDLOJA      = :CDLOJA   ) AND              
               (V.IDSITUVENDA = 'O'         ) AND               
               (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL) AND
               ((INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0) OR (:TCDPRODUTO = 'T')) AND
               (V.NRORG = :NRORG)              
               
            GROUP BY
               I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, DTENTRVENDA, TO_CHAR(DTENTRVENDA, 'D'), V.DTVENDA) A
        GROUP BY
           A.CDPRODUTO, A.CDARVPROD, A.NMPRODUTO, A.DTENTRVENDA, NRDIASEMANA, DATADAVENDA, DTVENDA
        ORDER BY
           A.CDARVPROD)
           GROUP BY CDPRODUTO, CDARVPROD, NRSEMANA, NRDIASEMANA, DTENTRVENDA, NMPRODUTO, TO_CHAR(DTVENDA, 'HH24')
           ORDER BY CDARVPROD, NRDIASEMANA, NRSEMANA, HORA";
           
    const INSERT_PREV_PROD_HR = "
       INSERT INTO PREVISAOVNDPRODHR (
        NRSEQPREVPDV,
        CDFILIAL,
        CDLOJA,
        CDPRODUTO,
        NRDIASEMANA,
        NRSEMANA,
        HORA,
        DTPRODCONPDV,
        QTCONSUMO,
        PERCENTUALHR,
        NRORG
    ) 
         
 SELECT DISTINCT
            PREV.NRSEQPREVPDV,
            :CDFILIAL CDFILIAL,
            :CDLOJA AS CDLOJA,
            PREV.CDPRODUTO,
            PREV.NRDIASEMANA,
            PREV.NRSEMANA,
            HIST.HORA,
            PREV.DTPRODCONPDV,
            ROUND((( SUM(HIST.QTCONSUMO) OVER(
                PARTITION BY HIST.CDPRODUTO, HIST.NRSEMANA, HIST.NRDIASEMANA, HIST.HORA) / SUM(HIST.QTCONSUMO) OVER(
                PARTITION BY HIST.CDPRODUTO, HIST.NRSEMANA, HIST.NRDIASEMANA
            ) ) * PREV.QTPRODCONPDVDIA) , 2) AS QTCONSUMO,
            ( SUM(HIST.QTCONSUMO) OVER(
                PARTITION BY HIST.CDPRODUTO, HIST.NRSEMANA, HIST.NRDIASEMANA, HIST.HORA) / SUM(HIST.QTCONSUMO) OVER(
                PARTITION BY HIST.CDPRODUTO, HIST.NRSEMANA, HIST.NRDIASEMANA
            ) ) AS PERCENTUALHR,
            
            :NRORG AS NRORG
        FROM
            (
                SELECT DISTINCT
                    CDPRODUTO,
                    CDARVPROD,
                    NMPRODUTO,
                    DTENTRVENDA,
                    NRDIASEMANA,
                    SUM(QTCONSUMO) QTCONSUMO,
                    SUM(QTCOMPRARSUG) QTCOMPRARSUG,
                    SUM(QTCOMPRAR) AS QTCOMPRAR,
                    NRSEMANA,
                    TO_CHAR(DTVENDA, 'HH24') AS HORA
                FROM
                    (
                        SELECT
                            A.CDPRODUTO,
                            A.CDARVPROD,
                            A.NMPRODUTO,
                            A.DTENTRVENDA,
                            SUM(A.QTCONSUMO) AS QTCONSUMO,
                            SUM(A.QTCOMPRARSUG) AS QTCOMPRARSUG,
                            SUM(A.QTCOMPRAR) AS QTCOMPRAR,
                            NRDIASEMANA,
                            DATADAVENDA,
                            CASE
                                WHEN TO_CHAR(DATADAVENDA, 'DD') <= 7 THEN
                                    '1'
                                WHEN TO_CHAR(DATADAVENDA, 'DD') > 7 AND TO_CHAR(DATADAVENDA, 'DD') <= 14 THEN
                                    '2'
                                WHEN TO_CHAR(DATADAVENDA, 'DD') > 14
                                     AND TO_CHAR(DATADAVENDA, 'DD') <= 21 THEN
                                    '3'
                                WHEN TO_CHAR(DATADAVENDA, 'DD') > 21
                                     AND TO_CHAR(DATADAVENDA, 'DD') <= 28 THEN
                                    '4'
                                WHEN TO_CHAR(DATADAVENDA, 'DD') > 28 THEN
                                    '5'
                            END AS NRSEMANA,
                            DTVENDA
                        FROM
                            (
                                SELECT
                                    I.CDPRODUTO,
                                    P.CDARVPROD,
                                    P.NMPRODUTO,
                                    TO_DATE(:DTINICIAL) AS DTENTRVENDA,
                                    SUM(I.QTPRODVEND) AS QTCONSUMO,
                                    DTENTRVENDA DATADAVENDA,
                                    SUM(I.QTPRODVEND) AS QTCOMPRARSUG,
                                    SUM(I.QTPRODVEND) AS QTCOMPRAR,
                                    TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA,
                                    V.DTVENDA
                                FROM
                                    VENDA       V
                                    JOIN ITEMVENDA   I ON ( V.CDFILIAL = I.CDFILIAL )
                                                        AND ( V.CDCAIXA = I.CDCAIXA )
                                                        AND ( V.NRSEQVENDA = I.NRSEQVENDA )
                                                        AND ( V.NRORG = I.NRORG )
                                    JOIN PRODUTO     P ON ( I.CDPRODUTO = P.CDPRODUTO )
                                                      AND ( V.NRORG = P.NRORG )
                                WHERE
                                    ( V.CDFILIAL = :CDFILIAL )
                                    AND ( V.CDLOJA = :CDLOJA )
                                    AND ( V.IDSITUVENDA = 'O' )
                                    AND ( V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL )
                                    AND ( ( INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0 )
                                          OR ( :TCDPRODUTO = 'T' ) )
                                    AND ( V.NRORG = :NRORG )
                                GROUP BY
                                    I.CDPRODUTO,
                                    P.CDARVPROD,
                                    P.NMPRODUTO,
                                    DTENTRVENDA,
                                    TO_CHAR(DTENTRVENDA, 'D'),
                                    V.DTVENDA
                                UNION
                                SELECT
                                    I.CDPRODUTO,
                                    P.CDARVPROD,
                                    P.NMPRODUTO,
                                    TO_DATE(:DTINICIAL) AS DTENTRVENDA,
                                    SUM(I.QTITVENDAEST) AS QTCONSUMO,
                                    DTENTRVENDA DATADAVENDA,
                                    SUM(I.QTITVENDAEST) AS QTCOMPRARSUG,
                                    SUM(I.QTITVENDAEST) AS QTCOMPRAR,
                                    TO_CHAR(DTENTRVENDA, 'D') NRDIASEMANA,
                                    V.DTVENDA
                                FROM
                                    VENDA          V
                                    JOIN ITEMVENDAEST   I ON ( V.CDFILIAL = I.CDFILIAL )
                                                           AND ( V.CDCAIXA = I.CDCAIXA )
                                                           AND ( V.NRSEQVENDA = I.NRSEQVENDA )
                                                           AND ( V.NRORG = I.NRORG )
                                    JOIN PRODUTO        P ON ( I.CDPRODUTO = P.CDPRODUTO )
                                                      AND ( V.NRORG = P.NRORG )
                                WHERE
                                    ( V.CDFILIAL = :CDFILIAL )
                                    AND ( V.CDLOJA = :CDLOJA )
                                    AND ( V.IDSITUVENDA = 'O' )
                                    AND ( V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL )
                                    AND ( ( INSTR(:CDPRODUTO, P.CDPRODUTO) <> 0 )
                                          OR ( :TCDPRODUTO = 'T' ) )
                                    AND ( V.NRORG = :NRORG )
                                GROUP BY
                                    I.CDPRODUTO,
                                    P.CDARVPROD,
                                    P.NMPRODUTO,
                                    DTENTRVENDA,
                                    TO_CHAR(DTENTRVENDA, 'D'),
                                    V.DTVENDA
                            ) A
                        GROUP BY
                            A.CDPRODUTO,
                            A.CDARVPROD,
                            A.NMPRODUTO,
                            A.DTENTRVENDA,
                            NRDIASEMANA,
                            DATADAVENDA,
                            DTVENDA
                        ORDER BY
                            A.CDARVPROD
                    )
                GROUP BY
                    CDPRODUTO,
                    CDARVPROD,
                    NRSEMANA,
                    NRDIASEMANA,
                    DTENTRVENDA,
                    NMPRODUTO,
                    TO_CHAR(DTVENDA, 'HH24')
                ORDER BY
                    CDARVPROD,
                    NRDIASEMANA,
                    NRSEMANA,
                    HORA
            ) HIST
             JOIN (SELECT CDPRODUTO, NRSEQPREVPDV, QTPRODCONPDVDIA, TO_CHAR(DTPRODCONPDV, 'D') AS NRDIASEMANA, DTPRODCONPDV,
                            CASE
                                WHEN TO_CHAR(DTPRODCONPDV, 'DD') <= 7 THEN
                                    '1'
                                WHEN TO_CHAR(DTPRODCONPDV, 'DD') > 7 AND TO_CHAR(DTPRODCONPDV, 'DD') <= 14 THEN
                                    '2'
                                WHEN TO_CHAR(DTPRODCONPDV, 'DD') > 14
                                     AND TO_CHAR(DTPRODCONPDV, 'DD') <= 21 THEN
                                    '3'
                                WHEN TO_CHAR(DTPRODCONPDV, 'DD') > 21
                                     AND TO_CHAR(DTPRODCONPDV, 'DD') <= 28 THEN
                                    '4'
                                WHEN TO_CHAR(DTPRODCONPDV, 'DD') > 28 THEN
                                    '5'
                            END AS NRSEMANA
                        FROM PREVCONSPDVDIA
                        WHERE  NRSEQPREVPDV = :NRSEQPREVPDV)
            
            PREV ON
                HIST.CDPRODUTO = PREV.CDPRODUTO AND
                HIST.NRDIASEMANA = PREV.NRDIASEMANA AND
                HIST.NRSEMANA = PREV.NRSEMANA               
                
                ORDER BY CDPRODUTO, NRSEMANA, NRDIASEMANA, HORA
    ";
    const GET_HIST_META = "
    SELECT DISTINCT
        MT.CDMETAPREV,
        MT.CDFILIAL,
        TO_CHAR(MT.DTINICIO, 'DD/MM/YYYY') AS DTINICIO,
        TO_CHAR(MT.DTFINAL, 'DD/MM/YYYY') AS DTFINAL,
        MT.VRPERCACRES,
        MT.CDOPERINCLUSAO CDOPERADOR,
        DTI.DTMES AS DTGERAINI,
        ((SELECT COUNT(*) FROM DATA WHERE ('01/'||TO_CHAR(DTF.DTMES, 'MM/YYYY')) = TO_CHAR(DTMES, 'DD/MM/YYYY') ) || '/' ||TO_CHAR(DTF.DTMES, 'MM/YYYY')) AS DTGERAFIN
        
    FROM
        METAPREVISAOHIST MT
        JOIN DATA DTI ON
            NVL(TO_CHAR(DTI.DTMES, 'MM'), '01') = MT.CDMESINICIAL AND 
            TO_CHAR(DTI.DTMES, 'YYYY') = MT.CDANO
        JOIN DATA DTF ON 
            NVL(TO_CHAR(DTF.DTMES, 'MM'), '12') = MT.CDMESFINAL AND 
            TO_CHAR(DTF.DTMES, 'YYYY') = MT.CDANO            
            WHERE MT.IDSTATUS ='P'
                    AND MT.CDFILIAL = :CDFILIAL 
        
    ";
    
    const UPD_METAPREVISAOHIST = "
    UPDATE METAPREVISAOHIST 
        SET IDSTATUS = 'C'
        WHERE CDFILIAL = :CDFILIAL
        AND CDMETAPREV = :CDMETAPREV
    ";
    
    const GET_MAT_PRIMA = 
    "SELECT
      PC.CDFILIAL || ' - ' || FI.NMFILIAL AS FILIAL, 
      PP.CDLOJA || ' - ' || LJ.NMLOJA AS LOJA,
      'Nr.Previsão: ' || PC.NRSEQPREVPDV || ' - Responsável: ' || PP.CDOPERADOR || ' - ' || OP.NMOPERADOR || ' - Período Previsão: ' || to_char(PP.DTINIPCONPDV,'DD/MM/YYYY') || ' até ' || to_char(PP.DTFINPCONPDV,'DD/MM/YYYY') AS INFO,
      PC.NRSEQPREVPDV, PC.CDFILIAL    , FI.NMFILIAL    , PP.CDLOJA      , LJ.NMLOJA      , 
      PP.CDOPERADOR  , OP.NMOPERADOR, PP.DTPREVCONPDV,
      PP.DTINIPCONPDV, PP.DTFINPCONPDV, PR.CDARVPROD   ,  PR.NMPRODUTO , PR.SGUNIDADE ,
      SUM(PC.QTPRODCONPDV) AS QTPRODCONPDV, SUM(PC.QTPRODCONINF) AS QTPRODCONINF,
      SUM(PC.QTPRODINFCOM) AS QTPRODINFCOM
    FROM
      PREVISAOPDV PP, PREVCONSPDV PC, FILIAL FI, PRODUTO PR, LOJA LJ, OPERADOR OP
    WHERE
      (PP.CDFILIAL      = PC.CDFILIAL     ) AND
      (PP.NRSEQPREVPDV  = PC.NRSEQPREVPDV ) AND 
      (PP.CDLOJA        = LJ.CDLOJA       ) AND
      (PP.CDFILIAL      = LJ.CDFILIAL     ) AND
      (PP.CDOPERADOR    = OP.CDOPERADOR   ) AND
      (PC.CDFILIAL      = :CDFILIAL     ) AND
      ((INSTR(:CDLOJA, PP.CDLOJA) > 0) OR (:TCDLOJA = 'T')) AND
      (TRUNC(PP.DTPREVCONPDV) BETWEEN :DATAINI AND :DATAFIN) AND
      (PC.CDFILIAL      = FI.CDFILIAL     ) AND
      (PC.CDPRODUTO     = PR.CDPRODUTO    ) AND
       ((PC.CDPRODUTO     <> PC.CDPRPAIPCPDV) OR (PC.CDPRPAIPCPDV IS NULL))
    GROUP BY
      PC.CDFILIAL    , FI.NMFILIAL    , PP.CDLOJA      , LJ.NMLOJA      , 
      PP.CDOPERADOR  , OP.NMOPERADOR  , PC.NRSEQPREVPDV, PR.CDARVPROD   ,  PR.NMPRODUTO  ,
      PP.DTINIPCONPDV, PP.DTFINPCONPDV,  
      PP.DTPREVCONPDV, PR.SGUNIDADE 
    ORDER BY
      PC.CDFILIAL    , FI.NMFILIAL    , PP.CDLOJA      , LJ.NMLOJA      , 
      PP.CDOPERADOR  , OP.NMOPERADOR  , PC.NRSEQPREVPDV,  PR.CDARVPROD  , PR.NMPRODUTO  ,
      PP.DTINIPCONPDV, PP.DTFINPCONPDV,  PP.DTPREVCONPDV
    ";
}